import { DataUrl } from '../util/DataUrl';
/**
 * Represents a base64 encoded binary
 */
export class EncodedBinary {
    constructor(_data, _mimeType) {
        this._data = _data;
        this._mimeType = _mimeType;
    }
    /**
     * Get the base64 encoded data
     * @returns {string}
     */
    get data() {
        return this._data;
    }
    /**
     * Get the mime-type
     * @returns {string|string}
     */
    get mimeType() {
        return this._mimeType || 'application/octet-stream';
    }
    /**
     * Returns a 'data url' representation of this binary, including the encoded data
     * @returns {string}
     */
    toUrl() {
        return DataUrl.createDataUrl(this.mimeType, this.data);
    }
}
